@@ -14,9 +14,37 @@
 -- Maintainer  : Ivan A. Malison
 -- Stability   : unstable
 -- Portability : unportable
+--
+-- The "System.Taffybar.Context" module provides the core functionality of the
+-- taffybar library. It gets its name from the 'Context' record, which stores
+-- runtime information and objects, which are used by many of the widgets that
+-- taffybar provides. 'Context' is typically accessed through the 'Reader'
+-- interface of 'TaffyIO'.
 -----------------------------------------------------------------------------
 
-module System.Taffybar.Context where
+module System.Taffybar.Context
+  ( Context(..)
+  , TaffybarConfig(..)
+  , Taffy
+  , TaffyIO
+  , BarConfig(..)
+  , BarConfigGetter
+  , appendHook
+  , buildContext
+  , buildEmptyContext
+  , defaultTaffybarConfig
+  , getState
+  , getStateDefault
+  , putState
+  , forceRefreshTaffyWindows
+  , refreshTaffyWindows
+  , runX11
+  , runX11Def
+  , subscribeToAll
+  , subscribeToPropertyEvents
+  , taffyFork
+  , unsubscribe
+  ) where
 
 import           Control.Arrow ((&&&))
 import           Control.Concurrent (forkIO)
@@ -29,10 +57,12 @@ import           Control.Monad.Trans.Maybe
 import           Control.Monad.Trans.Reader
 import qualified DBus.Client as DBus
 import           Data.Data
+import           Data.Default (Default(..))
 import           Data.GI.Base.ManagedPtr (unsafeCastTo)
 import           Data.Int
 import           Data.List
 import qualified Data.Map as M
+import qualified Data.Text as T
 import           Data.Tuple.Select
 import           Data.Tuple.Sequence
 import           Data.Unique
@@ -56,8 +86,13 @@ logIO = logM "System.Taffybar.Context"
 logC :: MonadIO m => System.Log.Logger.Priority -> String -> m ()
 logC p = liftIO . logIO p
 
-type Taffy m v = MonadIO m => ReaderT Context m v
+-- | 'Taffy' is a monad transformer that provides 'Reader' for 'Context'.
+type Taffy m v = ReaderT Context m v
+
+-- | 'TaffyIO' is 'IO' wrapped with a 'ReaderT' providing 'Context'. This is the
+-- type of most widgets and callback in taffybar.
 type TaffyIO v = ReaderT Context IO v
+
 type Listener = Event -> Taffy IO ()
 type SubscriptionList = [(Unique, Listener)]
 data Value = forall t. Typeable t => Value t
@@ -69,12 +104,20 @@ fromValue (Value v) =
   else
     Nothing
 
+-- | 'BarConfig' specifies the configuration for a single taffybar window.
 data BarConfig = BarConfig
-  { strutConfig :: StrutConfig
+  {
+  -- | The strut configuration to use for the bar
+    strutConfig :: StrutConfig
+  -- | The amount of spacing in pixels between bar widgets
   , widgetSpacing :: Int32
+  -- | Constructors for widgets that should be placed at the beginning of the bar.
   , startWidgets :: [TaffyIO Gtk.Widget]
+  -- | Constructors for widgets that should be placed in the center of the bar.
   , centerWidgets :: [TaffyIO Gtk.Widget]
+  -- | Constructors for widgets that should be placed at the end of the bar.
   , endWidgets :: [TaffyIO Gtk.Widget]
+  -- | A unique identifier for the bar, that can be used e.g. when toggling.
   , barId :: Unique
   }
 
@@ -83,27 +126,47 @@ instance Eq BarConfig where
 
 type BarConfigGetter = TaffyIO [BarConfig]
 
+-- | 'TaffybarConfig' provides an advanced interface for configuring taffybar.
+-- Through the 'getBarConfigsParam', it is possible to specify different
+-- taffybar configurations depending on the number of monitors present, and even
+-- to specify different taffybar configurations for each monitor.
 data TaffybarConfig = TaffybarConfig
-  { dbusClientParam :: Maybe DBus.Client
+  {
+  -- | An optional dbus client to use.
+    dbusClientParam :: Maybe DBus.Client
+  -- | Hooks that should be executed at taffybar startup.
   , startupHook :: TaffyIO ()
+  -- | A 'TaffyIO' action that returns a list of 'BarConfig' where each element
+  -- describes a taffybar window that should be spawned.
   , getBarConfigsParam :: BarConfigGetter
-  , cssPath :: Maybe FilePath
+  -- | A list of 'FilePath' each of which should be loaded as css files at
+  -- startup.
+  , cssPaths :: [FilePath]
+  -- | A field used (only) by dyre to provide an error message.
   , errorMsg :: Maybe String
   }
 
+
+-- | Append the provided 'TaffyIO' hook to the 'startupHook' of the given
+-- 'TaffybarConfig'.
 appendHook :: TaffyIO () -> TaffybarConfig -> TaffybarConfig
 appendHook hook config = config
   { startupHook = startupHook config >> hook }
 
+-- | Default values for a 'TaffybarConfig'. Not usuable without at least
+-- properly setting 'getBarConfigsParam'.
 defaultTaffybarConfig :: TaffybarConfig
 defaultTaffybarConfig = TaffybarConfig
   { dbusClientParam = Nothing
   , startupHook = return ()
   , getBarConfigsParam = return []
-  , cssPath = Nothing
+  , cssPaths = []
   , errorMsg = Nothing
   }
 
+instance Default TaffybarConfig where
+  def = defaultTaffybarConfig
+
 -- | A "Context" value holds all of the state associated with a single running
 -- instance of taffybar. It is typically accessed from a widget constructor
 -- through the "TaffyIO" monad transformer stack.
@@ -113,25 +176,25 @@ data Context = Context
     x11ContextVar :: MV.MVar X11Context
   -- | The handlers which will be evaluated against incoming X11 events.
   , listeners :: MV.MVar SubscriptionList
-  -- | A collection of miscellaneous peices of state which are keyed by their
-  -- types. Most new peices of state should go here, rather than in a new field
-  -- in "Context". State stored here is typically accessed through
-  -- "getStateDefault".
+  -- | A collection of miscellaneous pieces of state which are keyed by their
+  -- types. Most new pieces of state should go here, rather than in a new field
+  -- in 'Context'. State stored here is typically accessed through
+  -- 'getStateDefault'.
   , contextState :: MV.MVar (M.Map TypeRep Value)
   -- | Used to track the windows that taffybar is currently controlling, and
-  -- which "BarConfig" objects they are associated with.
+  -- which 'BarConfig' objects they are associated with.
   , existingWindows :: MV.MVar [(BarConfig, Gtk.Window)]
-  -- | The shared user session "DBus.Client".
+  -- | The shared user session 'DBus.Client'.
   , sessionDBusClient :: DBus.Client
-  -- | The shared system session "DBus.Client".
+  -- | The shared system session 'DBus.Client'.
   , systemDBusClient :: DBus.Client
   -- | The action that will be evaluated to get the bar configs associated with
   -- each active monitor taffybar should run on.
   , getBarConfigs :: BarConfigGetter
   -- | Populated with the BarConfig that resulted in the creation of a given
   -- widget, when its constructor is called. This lets widgets access thing like
-  -- who their neighbors are. Note that the value of "contextBarConfig" is
-  -- different for widgets belonging to bar windows on differnt monitors.
+  -- who their neighbors are. Note that the value of 'contextBarConfig' is
+  -- different for widgets belonging to bar windows on different monitors.
   , contextBarConfig :: Maybe BarConfig
   }
 
@@ -178,8 +241,11 @@ buildContext TaffybarConfig
   logIO DEBUG "Context build finished"
   return context
 
+-- | Build an empty taffybar context. This function is mostly useful for
+-- invoking functions that yield 'TaffyIO' values in a testing setting (e.g. in
+-- a repl).
 buildEmptyContext :: IO Context
-buildEmptyContext = buildContext defaultTaffybarConfig
+buildEmptyContext = buildContext def
 
 buildBarWindow :: Context -> BarConfig -> IO Gtk.Window
 buildBarWindow context barConfig = do
@@ -189,9 +255,16 @@ buildBarWindow context barConfig = do
       show $ strutConfig barConfig
 
   window <- Gtk.windowNew Gtk.WindowTypeToplevel
-  box <- Gtk.boxNew Gtk.OrientationHorizontal $ fromIntegral $ widgetSpacing barConfig
+  box <- Gtk.boxNew Gtk.OrientationHorizontal $ fromIntegral $
+         widgetSpacing barConfig
   _ <- widgetSetClassGI box "taffy-box"
-  centerBox <- Gtk.boxNew Gtk.OrientationHorizontal $ fromIntegral $ widgetSpacing barConfig
+  centerBox <- Gtk.boxNew Gtk.OrientationHorizontal $
+               fromIntegral $ widgetSpacing barConfig
+
+  _ <- widgetSetClassGI centerBox "center-box"
+  Gtk.widgetSetVexpand centerBox True
+  Gtk.setWidgetValign centerBox Gtk.AlignFill
+  Gtk.setWidgetHalign centerBox Gtk.AlignCenter
   Gtk.boxSetCenterWidget box (Just centerBox)
 
   setupStrutWindow (strutConfig barConfig) window
@@ -199,18 +272,24 @@ buildBarWindow context barConfig = do
 
   _ <- widgetSetClassGI window "taffy-window"
 
-  let addWidgetWith widgetAdd buildWidget =
-        runReaderT buildWidget thisContext >>= widgetAdd
-      addToStart widget = Gtk.boxPackStart box widget False False 0
-      addToEnd widget = Gtk.boxPackEnd box widget False False 0
-      addToCenter widget = Gtk.boxPackStart centerBox widget False False 0
+  let addWidgetWith widgetAdd (count, buildWidget) =
+        runReaderT buildWidget thisContext >>= widgetAdd count
+      addToStart count widget = do
+        _ <- widgetSetClassGI widget $ T.pack $ printf "left-%d" (count :: Int)
+        Gtk.boxPackStart box widget False False 0
+      addToEnd count widget = do
+        _ <- widgetSetClassGI widget $ T.pack $ printf "right-%d" (count :: Int)
+        Gtk.boxPackEnd box widget False False 0
+      addToCenter count widget = do
+        _ <- widgetSetClassGI widget $ T.pack $ printf "center-%d" (count :: Int)
+        Gtk.boxPackStart centerBox widget False False 0
 
   logIO DEBUG "Building start widgets"
-  mapM_ (addWidgetWith addToStart) (startWidgets barConfig)
+  mapM_ (addWidgetWith addToStart) $ zip [1..] (startWidgets barConfig)
   logIO DEBUG "Building center widgets"
-  mapM_ (addWidgetWith addToCenter) (centerWidgets barConfig)
+  mapM_ (addWidgetWith addToCenter) $ zip [1..] (centerWidgets barConfig)
   logIO DEBUG "Building end widgets"
-  mapM_ (addWidgetWith addToEnd) (endWidgets barConfig)
+  mapM_ (addWidgetWith addToEnd) $ zip [1..] (endWidgets barConfig)
 
   makeWindowTransparent window
 
@@ -276,6 +355,8 @@ refreshTaffyWindows = liftReader postGUIASync $ do
   logC DEBUG "Finished refreshing windows"
   return ()
 
+-- | Forcibly refresh taffybar windows, even if there are existing windows that
+-- correspond to the uniques in the bar configs yielded by 'barConfigGetter'.
 forceRefreshTaffyWindows :: TaffyIO ()
 forceRefreshTaffyWindows =
   asks existingWindows >>= lift . flip MV.modifyMVar_ deleteWindows >>
@@ -288,21 +369,26 @@ forceRefreshTaffyWindows =
 asksContextVar :: (r -> MV.MVar b) -> ReaderT r IO b
 asksContextVar getter = asks getter >>= lift . MV.readMVar
 
+-- | Run a function needing an X11 connection in 'TaffyIO'.
 runX11 :: X11Property a -> TaffyIO a
 runX11 action =
   asksContextVar x11ContextVar >>= lift . runReaderT action
 
+-- | Use 'runX11' together with 'postX11RequestSyncProp' on the provided
+-- property. Return the provided default if 'Nothing' is returned
+-- 'postX11RequestSyncProp'.
 runX11Def :: a -> X11Property a -> TaffyIO a
-runX11Def def prop = runX11 $ postX11RequestSyncProp prop def
+runX11Def dflt prop = runX11 $ postX11RequestSyncProp prop dflt
 
 runX11Context :: MonadIO m => Context -> a -> X11Property a -> m a
-runX11Context context def prop =
-  liftIO $ runReaderT (runX11Def def prop) context
+runX11Context context dflt prop =
+  liftIO $ runReaderT (runX11Def dflt prop) context
 
+-- | Get a state value by type from the 'contextState' field of 'Context'.
 getState :: forall t. Typeable t => Taffy IO (Maybe t)
 getState = do
   stateMap <- asksContextVar contextState
-  let maybeValue = M.lookup (typeOf (undefined :: t)) stateMap
+  let maybeValue = M.lookup (typeRep (Proxy :: Proxy t)) stateMap
   return $ maybeValue >>= fromValue
 
 -- | Like "putState", but avoids aquiring a lock if the value is already in the
@@ -319,7 +405,7 @@ putState getValue = do
   contextVar <- asks contextState
   ctx <- ask
   lift $ MV.modifyMVar contextVar $ \contextStateMap ->
-    let theType = typeOf (undefined :: t)
+    let theType = typeRep (Proxy :: Proxy t)
         currentValue = M.lookup theType contextStateMap
         insertAndReturn value =
           (M.insert theType (Value value) contextStateMap, value)
