@@ -7,6 +7,8 @@
 -- Maintainer  : Ivan A. Malison
 -- Stability   : unstable
 -- Portability : unportable
+--
+-- This module provides various startup hooks that can be added to 'TaffyConfig'.
 -----------------------------------------------------------------------------
 
 module System.Taffybar.Hooks
@@ -35,34 +37,45 @@ import           System.Environment.XDG.DesktopEntry
 import           System.Taffybar.LogFormatter
 import           System.Taffybar.Util
 
-newtype NetworkInfoChan = NetworkInfoChan (BroadcastChan In [(String, (Rational, Rational))])
+-- | The type of the channel that provides network information in taffybar.
+newtype NetworkInfoChan =
+  NetworkInfoChan (BroadcastChan In [(String, (Rational, Rational))])
 
-buildInfoChan :: Double -> IO NetworkInfoChan
-buildInfoChan interval = do
+-- | Build a 'NetworkInfoChan' that refreshes at the provided interval.
+buildNetworkInfoChan :: Double -> IO NetworkInfoChan
+buildNetworkInfoChan interval = do
   chan <- newBroadcastChan
   _ <- forkIO $ monitorNetworkInterfaces interval (void . writeBChan chan)
   return $ NetworkInfoChan chan
 
+-- | Get the 'NetworkInfoChan' from 'Context', creating it if it does not exist.
 getNetworkChan :: TaffyIO NetworkInfoChan
-getNetworkChan = getStateDefault $ lift $ buildInfoChan 2.0
+getNetworkChan = getStateDefault $ lift $ buildNetworkInfoChan 2.0
 
+-- | Set the log formatter used in the taffybar process
 setTaffyLogFormatter :: String -> IO ()
 setTaffyLogFormatter loggerName = do
   handler <- taffyLogHandler
   updateGlobalLogger loggerName $ setHandlers [handler]
 
+-- | Add 'refreshrefreshBatteriesOnPropChange' to the 'startupHook' of the
+-- provided 'TaffybarConfig'.
 withBatteryRefresh :: TaffybarConfig -> TaffybarConfig
 withBatteryRefresh = appendHook refreshBatteriesOnPropChange
 
+-- | Load the 'DesktopEntry' cache from 'Context' state.
 getDirectoryEntriesByClassName :: TaffyIO (MM.MultiMap String DesktopEntry)
 getDirectoryEntriesByClassName =
   getStateDefault readDirectoryEntriesDefault
 
+-- | Update the 'DesktopEntry' cache every 60 seconds.
 updateDirectoryEntriesCache :: TaffyIO ()
 updateDirectoryEntriesCache = ask >>= \ctx ->
   void $ lift $ foreverWithDelay (60 :: Double) $ flip runReaderT ctx $
-       putState readDirectoryEntriesDefault
+       void $ putState readDirectoryEntriesDefault
 
+-- | Read 'DesktopEntry' values into a 'MM.Multimap', where they are indexed by
+-- the class name specified in the 'DesktopEntry'.
 readDirectoryEntriesDefault :: TaffyIO (MM.MultiMap String DesktopEntry)
 readDirectoryEntriesDefault = lift $
   indexDesktopEntriesByClassName <$> getDirectoryEntriesDefault
