@@ -1,10 +1,20 @@
 {-# LANGUAGE OverloadedStrings #-}
--- | This is a simple library to query the Linux UPower daemon (via DBus) for
--- battery information.
+-----------------------------------------------------------------------------
+-- |
+-- Module      : System.Taffybar.Information.Battery
+-- Copyright   : (c) Ivan A. Malison
+-- License     : BSD3-style (see LICENSE)
+--
+-- Maintainer  : Ivan A. Malison
+-- Stability   : unstable
+-- Portability : unportable
+--
+-- This module provides functions for querying battery information using the
+-- UPower dbus, as well as a "BroadcastChan" system for allowing multiple
+-- readers to receive 'BatteryState' updates without duplicating requests.
+-----------------------------------------------------------------------------
 module System.Taffybar.Information.Battery
-  (
-  -- * Types
-    BatteryInfo(..)
+  ( BatteryInfo(..)
   , BatteryState(..)
   , BatteryTechnology(..)
   , BatteryType(..)
@@ -201,7 +211,8 @@ registerForUPowerPropertyChanges properties signalHandler = do
 
 -- | Monitor the DisplayDevice for changes, writing a new "BatteryInfo" object
 -- to returned "MVar" and "Chan" objects
-monitorDisplayBattery :: [String] -> TaffyIO (BroadcastChan In BatteryInfo, MVar BatteryInfo)
+monitorDisplayBattery ::
+  [String] -> TaffyIO (BroadcastChan In BatteryInfo, MVar BatteryInfo)
 monitorDisplayBattery propertiesToMonitor = do
   lift $ batteryLog DEBUG "Starting Battery Monitor"
   client <- asks systemDBusClient
