@@ -2,7 +2,7 @@
 -- | This is a graph widget inspired by the widget of the same name in Awesome
 -- (the window manager). It plots a series of data points similarly to a bar
 -- graph. This version must be explicitly fed data with 'graphAddSample'. For a
--- more automated version, see 'PollingGraph'.
+-- more automated version, see "System.Taffybar.Widgets.Generic.PollingGraph".
 --
 -- Like Awesome, this graph can plot multiple data sets in one widget. The data
 -- sets are plotted in the order provided by the caller.
@@ -23,6 +23,7 @@ module System.Taffybar.Widget.Generic.Graph (
 import           Control.Concurrent
 import           Control.Monad ( when )
 import           Control.Monad.IO.Class
+import           Data.Default ( Default(..) )
 import           Data.Foldable ( mapM_ )
 import           Data.Sequence ( Seq, (<|), viewl, ViewL(..) )
 import qualified Data.Sequence as S
@@ -45,6 +46,7 @@ data GraphState =
 
 data GraphDirection = LEFT_TO_RIGHT | RIGHT_TO_LEFT deriving (Eq)
 
+-- 'RGBA' represents a color with a transparency.
 type RGBA = (Double, Double, Double, Double)
 
 -- | The style of the graph. Generally, you will want to draw all 'Area' graphs
@@ -93,6 +95,9 @@ defaultGraphConfig =
   , graphDirection = LEFT_TO_RIGHT
   }
 
+instance Default GraphConfig where
+  def = defaultGraphConfig
+
 -- | Add a data point to the graph for each of the tracked data sets. There
 -- should be as many values in the list as there are data sets.
 graphAddSample :: GraphHandle -> [Double] -> IO ()
@@ -231,16 +236,21 @@ graphNew cfg = liftIO $ do
        (drawGraph mv drawArea) ctx >> return True
   box <- Gtk.boxNew Gtk.OrientationHorizontal 1
 
-  case graphLabel cfg of
-    Nothing  -> return ()
-    Just lbl -> do
-      l <- Gtk.labelNew (Nothing :: Maybe T.Text)
-      Gtk.labelSetMarkup l lbl
-      Gtk.boxPackStart box l False False 0
 
   Gtk.widgetSetVexpand drawArea True
   Gtk.widgetSetVexpand box True
   Gtk.boxPackStart box drawArea True True 0
-  Gtk.widgetShowAll box
-  giBox <- Gtk.toWidget box
-  return (giBox, GH mv)
+
+  widget <- case graphLabel cfg of
+    Nothing  -> Gtk.toWidget box
+    Just labelText -> do
+      overlay <- Gtk.overlayNew
+      label <- Gtk.labelNew Nothing
+      Gtk.labelSetMarkup label labelText
+      Gtk.containerAdd overlay box
+      Gtk.overlayAddOverlay overlay label
+      Gtk.toWidget overlay
+
+  Gtk.widgetShowAll widget
+
+  return (widget, GH mv)
