@@ -17,16 +17,18 @@ import           Control.Applicative
 import           Control.Arrow ((&&&))
 import           Control.Concurrent
 import qualified Control.Concurrent.MVar as MV
+import           Control.Exception.Enclosed (catchAny)
 import           Control.Monad
 import           Control.Monad.IO.Class
 import           Control.Monad.Trans.Class
 import           Control.Monad.Trans.Maybe
 import           Control.Monad.Trans.Reader
 import           Control.RateLimit
+import           Data.Default (Default(..))
 import qualified Data.Foldable as F
 import           Data.GI.Base.ManagedPtr (unsafeCastTo)
 import           Data.Int
-import           Data.List (intersect, sortBy, (\\))
+import           Data.List (elemIndex, intersect, sortBy, (\\))
 import qualified Data.Map as M
 import           Data.Maybe
 import qualified Data.MultiMap as MM
@@ -48,7 +50,6 @@ import           System.Taffybar.Information.EWMHDesktopInfo
 import           System.Taffybar.Information.SafeX11
 import           System.Taffybar.Information.X11DesktopInfo
 import           System.Taffybar.Util
-import           System.Taffybar.Widget.Decorators
 import           System.Taffybar.Widget.Generic.AutoSizeImage (autoSizeImage)
 import           System.Taffybar.Widget.Util
 import           System.Taffybar.WindowIcon
@@ -100,7 +101,7 @@ liftContext :: TaffyIO a -> WorkspacesIO a
 liftContext action = asks taffyContext >>= lift . runReaderT action
 
 liftX11Def :: a -> X11Property a -> WorkspacesIO a
-liftX11Def def prop = liftContext $ runX11Def def prop
+liftX11Def dflt prop = liftContext $ runX11Def dflt prop
 
 setWorkspaceWidgetStatusClass ::
      (MonadIO m, Gtk.IsWidget a) => Workspace -> a -> m ()
@@ -150,8 +151,6 @@ data WorkspacesConfig =
   WorkspacesConfig
   { widgetBuilder :: ControllerConstructor
   , widgetGap :: Int
-  , underlineHeight :: Int
-  , underlinePadding :: Int
   , maxIcons :: Maybe Int
   , minIcons :: Int
   , getWindowIconPixbuf :: WindowIconPixbufGetter
@@ -169,8 +168,6 @@ defaultWorkspacesConfig =
   WorkspacesConfig
   { widgetBuilder = buildButtonController defaultBuildContentsController
   , widgetGap = 0
-  , underlineHeight = 4
-  , underlinePadding = 1
   , maxIcons = Nothing
   , minIcons = 0
   , getWindowIconPixbuf = defaultGetWindowIconPixbuf
@@ -183,6 +180,9 @@ defaultWorkspacesConfig =
   , urgentWorkspaceState = False
   }
 
+instance Default WorkspacesConfig where
+  def = defaultWorkspacesConfig
+
 hideEmpty :: Workspace -> Bool
 hideEmpty Workspace { workspaceState = Empty } = False
 hideEmpty _ = True
@@ -457,6 +457,24 @@ onIconsChanged context = rateLimitFn context onIconsChanged' combineRequests
              wLog DEBUG (printf "Updating %s icons." $ show idx) >>
              updateWidget c (IconUpdate $ Set.toList wids))
 
+initializeWWC ::
+  WorkspaceWidgetController a => a -> Workspace -> ReaderT WorkspacesContext IO WWC
+initializeWWC controller ws =
+  WWC <$> updateWidget controller (WorkspaceUpdate ws)
+
+-- | A WrappingController can be used to wrap some child widget with another
+-- abitrary widget.
+data WrappingController = WrappingController
+  { wrappedWidget :: Gtk.Widget
+  , wrappedController :: WWC
+  }
+
+instance WorkspaceWidgetController WrappingController where
+  getWidget = lift . Gtk.toWidget . wrappedWidget
+  updateWidget wc update = do
+    updated <- updateWidget (wrappedController wc) update
+    return wc { wrappedController = updated }
+
 data WorkspaceContentsController = WorkspaceContentsController
   { containerWidget :: Gtk.Widget
   , contentsControllers :: [WWC]
@@ -477,12 +495,59 @@ buildContentsController constructors ws = do
       { containerWidget = widget
       , contentsControllers = controllers
       }
-  WWC <$> updateWidget tempController (WorkspaceUpdate ws)
+  initializeWWC tempController ws
 
 defaultBuildContentsController :: ControllerConstructor
 defaultBuildContentsController =
   buildContentsController [buildLabelController, buildIconController]
 
+bottomLeftAlignedBoxWrapper :: T.Text -> ControllerConstructor -> ControllerConstructor
+bottomLeftAlignedBoxWrapper boxClass constructor ws = do
+  controller <- constructor ws
+  widget <- getWidget controller
+  ebox <- Gtk.eventBoxNew
+  _ <- widgetSetClassGI ebox boxClass
+  Gtk.widgetSetHalign ebox Gtk.AlignStart
+  Gtk.widgetSetValign ebox Gtk.AlignEnd
+  Gtk.containerAdd ebox widget
+  wrapped <- Gtk.toWidget ebox
+  let wrappingController = WrappingController
+                           { wrappedWidget = wrapped
+                           , wrappedController = controller
+                           }
+  initializeWWC wrappingController ws
+
+buildLabelOverlayController :: ControllerConstructor
+buildLabelOverlayController =
+  buildOverlayContentsController
+  [buildIconController]
+  [bottomLeftAlignedBoxWrapper "overlay-box" buildLabelController]
+
+buildOverlayContentsController ::
+  [ControllerConstructor] -> [ControllerConstructor] -> ControllerConstructor
+buildOverlayContentsController mainConstructors overlayConstructors ws = do
+  controllers <- mapM ($ ws) mainConstructors
+  overlayControllers <- mapM ($ ws) overlayConstructors
+  ctx <- ask
+  tempController <- lift $ do
+    mainContents <- Gtk.boxNew Gtk.OrientationHorizontal 0
+    mapM_ (flip runReaderT ctx . getWidget >=> Gtk.containerAdd mainContents)
+        controllers
+    outerBox <- Gtk.toWidget mainContents >>= buildPadBox
+    _ <- widgetSetClassGI mainContents "contents"
+    overlay <- Gtk.overlayNew
+    Gtk.containerAdd overlay outerBox
+    mapM_ (flip runReaderT ctx . getWidget >=>
+                Gtk.overlayAddOverlay overlay) overlayControllers
+
+    widget <- Gtk.toWidget overlay
+    return
+      WorkspaceContentsController
+      { containerWidget = widget
+      , contentsControllers = controllers ++ overlayControllers
+      }
+  initializeWWC tempController ws
+
 instance WorkspaceWidgetController WorkspaceContentsController where
   getWidget = return . containerWidget
   updateWidget cc update = do
@@ -505,7 +570,7 @@ buildLabelController ws = do
     lbl <- Gtk.labelNew Nothing
     _ <- widgetSetClassGI lbl "workspace-label"
     return LabelController { label = lbl }
-  WWC <$> updateWidget tempController (WorkspaceUpdate ws)
+  initializeWWC tempController ws
 
 instance WorkspaceWidgetController LabelController where
   getWidget = lift . Gtk.toWidget . label
@@ -586,7 +651,7 @@ buildIconController ws = do
       return
         IconController
         {iconsContainer = hbox, iconImages = [], iconWorkspace = ws}
-  WWC <$> updateWidget tempController (WorkspaceUpdate ws)
+  initializeWWC tempController ws
 
 instance WorkspaceWidgetController IconController where
   getWidget = lift . Gtk.toWidget . iconsContainer
@@ -617,16 +682,22 @@ constantScaleWindowIconPixbufGetter ::
 constantScaleWindowIconPixbufGetter constantSize getter =
   const $ scaledWindowIconPixbufGetter getter constantSize
 
+handleIconGetterException :: WindowIconPixbufGetter -> WindowIconPixbufGetter
+handleIconGetterException getter =
+  \size windowData -> catchAny (getter size windowData) $ \e -> do
+    wLog WARNING $ printf "Failed to get window icon for %s: %s" (show windowData) (show e)
+    return Nothing
+
 getWindowIconPixbufFromEWMH :: WindowIconPixbufGetter
-getWindowIconPixbufFromEWMH size windowData =
+getWindowIconPixbufFromEWMH = handleIconGetterException $ \size windowData ->
   runX11Def Nothing (getIconPixBufFromEWMH size $ windowId windowData)
 
 getWindowIconPixbufFromClass :: WindowIconPixbufGetter
-getWindowIconPixbufFromClass size windowData =
+getWindowIconPixbufFromClass = handleIconGetterException $ \size windowData ->
   lift $ getWindowIconFromClasses size (windowClass windowData)
 
 getWindowIconPixbufFromDesktopEntry :: WindowIconPixbufGetter
-getWindowIconPixbufFromDesktopEntry size windowData =
+getWindowIconPixbufFromDesktopEntry = handleIconGetterException $ \size windowData ->
   getWindowIconFromDesktopEntryByClasses size (windowClass windowData)
 
 getWindowIconPixbufFromChrome :: WindowIconPixbufGetter
@@ -667,6 +738,7 @@ addCustomIconsAndFallback getCustomIconPath fallback defaultGetter =
       lift $
       maybe (return Nothing) getPixbufFromFilePath $ getCustomIconPath wdata
 
+-- | Sort windows by top-left corner position.
 sortWindowsByPosition :: [WindowData] -> WorkspacesIO [WindowData]
 sortWindowsByPosition wins = do
   let getGeometryWorkspaces w = getDisplay >>= liftIO . (`safeGetGeometry` w)
@@ -684,6 +756,16 @@ sortWindowsByPosition wins = do
           (windowMinimized b, getLeftPos b)
   return $ sortBy compareWindowData wins
 
+-- | Sort windows in reverse _NET_CLIENT_LIST_STACKING order.
+-- Starting in xmonad-contrib 0.17.0, this is effectively focus history, active first.
+-- Previous versions erroneously stored focus-sort-order in _NET_CLIENT_LIST.
+sortWindowsByStackIndex :: [WindowData] -> WorkspacesIO [WindowData]
+sortWindowsByStackIndex wins = do
+  stackingWindows <- liftX11Def [] getWindowsStacking
+  let getStackIdx wd = fromMaybe (-1) $ elemIndex (windowId wd) stackingWindows
+      compareWindowData a b = compare (getStackIdx b) (getStackIdx a)
+  return $ sortBy compareWindowData wins
+
 updateImages :: IconController -> Workspace -> WorkspacesIO [IconWidget]
 updateImages ic ws = do
   WorkspacesContext {workspacesConfig = cfg} <- ask
@@ -749,9 +831,11 @@ updateIconWidget _ IconWidget
                    , iconForceUpdate = updateIcon
                    } windowData = do
   let statusString = maybe "inactive" getWindowStatusString windowData :: T.Text
+      title = T.pack . windowTitle <$> windowData
       setIconWidgetProperties =
         updateWidgetClasses iconButton [statusString] possibleStatusStrings
   void $ updateVar windowRef $ const $ return windowData
+  Gtk.widgetSetTooltipText iconButton title
   lift $ updateIcon >> setIconWidgetProperties
 
 data WorkspaceButtonController = WorkspaceButtonController
@@ -791,7 +875,7 @@ buildButtonController contentsBuilder workspace = do
     return $
       WWC
         WorkspaceButtonController
-        {button = ebox, buttonWorkspace = workspace, contentsController = cc}
+        { button = ebox, buttonWorkspace = workspace, contentsController = cc }
 
 switch :: (MonadIO m) => WorkspacesContext -> WorkspaceId -> m Bool
 switch ctx idx = do
